import json
from django import template
from django.utils.safestring import mark_safe

register = template.Library()

@register.simple_tag
def tablefield_render(value):
    if not value:
        return ""
    value = json.loads(value)
    data = value.get('data', None)
    if not data:
        return ""

    first_row_is_table_header = value.get('first_row_is_table_header', False)
    merge_сells = value.get('mergeCells', [])
    cells_meta = value.get('cellsMeta', [])

    data_objects = []
    
    
    # iterate over all table [['1', None, None], ['1', '2', '3']]
    # and replace values with objects with extra data - mergeCells and cellsMeta.    
    # and drop next cells when colspan or rowspan exists.
    colspan_count = 0
    rowspan_count = 0

    for ridx, row in enumerate(data):
        colspan_count += rowspan_count
        rowspan_count = 0
        for cidx, col in enumerate(row):
            colspan, rowspan = 0, 0

            if cidx == 0:
                data_objects.append([])

            # Drop next cells when colspan or rowspan exists.
            if colspan_count > 0 and not col:
                colspan_count = colspan_count - 1
                continue
            
            # find by cells position (row, col) in merge_cells
            merge_сell = next((x for x in merge_сells if x['row'] == ridx and x['col'] == cidx), None)
            if merge_сell:
                colspan = merge_сell.get('colspan', 0)
                colspan_count += colspan - 1
                rowspan = merge_сell.get('rowspan', 0)
                rowspan_count += rowspan - 1

            # find by cells position (row, col) in cells_meta
            cell_meta = next((x for x in cells_meta if x['row'] == ridx and x['col'] == cidx), None)
            class_name = cell_meta['val'] if cell_meta else None
            
            data_objects[ridx].append({
                'value': col,
                'colspan': colspan,
                'rowspan': rowspan,
                'className': class_name
                })


    table_header = data_objects.pop(0) if first_row_is_table_header else None

    t = template.loader.get_template("tablefield/table.html")
    return mark_safe(t.render(dict(
        data=data_objects,
        table_header=table_header,
        first_col_is_header=value.get('first_col_is_header', False),
        merge_сells=merge_сells,
        cells_meta=cells_meta
        )))


import json
from django.db.models import TextField
from django.forms.widgets import Widget
from django.contrib.admin.options import FORMFIELD_FOR_DBFIELD_DEFAULTS

DEFAULT_TABLE_OPTIONS = {
    'minSpareRows': 0,
    'startRows': 3,
    'startCols': 3,
    'colHeaders': False,
    'rowHeaders': False,
    'mergeCells': True,
    'contextMenu': [
        'row_above',
        'row_below',
        '---------',
        'col_left',
        'col_right',
        '---------',
        'remove_row',
        'remove_col',
        '---------',
        'mergeCells',
        '---------',
        'alignment',
        '---------',
        'undo',
        'redo'
    ],
    'editor': 'text',
    'stretchH': 'all',
    'height': 200,
    'renderer': 'html',
    'autoColumnSize': False,
}


class TableField(TextField):
    def __init__(self, *args, **kwargs):
        kwargs['blank'] = True
        kwargs['default'] = dict
        self.table_options = kwargs.pop('table_options', DEFAULT_TABLE_OPTIONS)
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        widget_class = kwargs.get('widget', TableFieldWidget)
        attrs = {}
        attrs["table_options_json"] = json.dumps(self.table_options)
        defaults = {
            'widget': widget_class(attrs=attrs),
        }
        return super().formfield(**defaults)

    def get_prep_value(self, value):
        return str(json.dumps(value))

    def from_db_value(self, value, expression, connection):
        return value

    def to_python(self, value):
        return json.loads(value)


class TableFieldWidget(Widget):
    template_name = 'tablefield/table_field.html'
    
    class Media:
        js = (
            '/static/tablefield/handsontable/handsontable.min.js',
            '/static/tablefield/table.js',
        )
        css = {
            'all': ('/static/tablefield/handsontable/handsontable.min.css',)
            }

FORMFIELD_FOR_DBFIELD_DEFAULTS[TableField] = {
        'widget': TableFieldWidget
    }


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django-tablefield",
    version="0.0.1b",
    author="Yury Lapshinov",
    author_email="y.raagin@gmail.com",
    description="tables in django admin",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/raagin/django-tablefield",
    packages=setuptools.find_packages(),
    include_package_data=True,
    license='MIT',
    zip_safe=False,
    python_requires='>=3',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)